'use client';
import { createContext, useContext, useEffect, useState } from 'react';
import { createClient } from '@/utils/supabase/client';
import type { Session, User as SupabaseUser } from '@supabase/supabase-js';
import type { User } from '@/lib/types';
import { useRouter } from 'next/navigation';

interface AuthContextType {
  session: Session | null;
  supabaseUser: SupabaseUser | null;
  user: User | null;
  loading: boolean;
  signOut: () => Promise<void>;
}

const AuthContext = createContext<AuthContextType | undefined>(undefined);

export const AuthProvider = ({ children }: { children: React.ReactNode }) => {
  const supabase = createClient();
  const router = useRouter();
  const [session, setSession] = useState<Session | null>(null);
  const [supabaseUser, setSupabaseUser] = useState<SupabaseUser | null>(null);
  const [user, setUser] = useState<User | null>(null);
  const [loading, setLoading] = useState(true);

  // Function to fetch user profile data
  const fetchUserProfile = async (userId: string) => {
    try {
      const { data: userData, error } = await supabase
        .from('profiles')
        .select('*')
        .eq('id', userId)
        .single();
        
      if (error) {
        console.error('Error fetching user profile:', error);
        return null;
      }
      
      return userData as User;
    } catch (error) {
      console.error('Error in fetchUserProfile:', error);
      return null;
    }
  };

  useEffect(() => {
    // Function to fetch authenticated user and session
    async function fetchAuthUser() {
      try {
        setLoading(true);
        
        // Get authenticated user - this validates with the server
        const { data: { user: authUser }, error: userError } = await supabase.auth.getUser();
        
        if (userError) {
          console.error('Error fetching authenticated user:', userError);
          setSupabaseUser(null);
          setSession(null);
          setUser(null);
          setLoading(false);
          return;
        }
        
        setSupabaseUser(authUser);
        
        // If we have an authenticated user, also get the session
        if (authUser) {
          const { data: { session }, error: sessionError } = await supabase.auth.getSession();
          
          if (sessionError) {
            console.error('Error fetching session:', sessionError);
          } else {
            setSession(session);
          }
          
          // Fetch user profile data
          const profileData = await fetchUserProfile(authUser.id);
          setUser(profileData);
        } else {
          setSession(null);
          setUser(null);
        }
      } catch (error) {
        console.error('Error in fetchAuthUser:', error);
      } finally {
        setLoading(false);
      }
    }

    // Initial fetch
    fetchAuthUser().catch((error) => console.error(error));

    // Set up auth state change listener
    const { data: { subscription } } = supabase.auth.onAuthStateChange(
      async (event, session) => {
        console.log('Auth state changed:', event);
        setLoading(true);
        
        if (session) {
          // Get authenticated user to validate the session
          const { data: { user: authUser }, error } = await supabase.auth.getUser();
          
          if (error || !authUser) {
            console.error('Error validating user after auth state change:', error);
            setSupabaseUser(null);
            setSession(null);
            setUser(null);
          } else {
            setSupabaseUser(authUser);
            setSession(session);
            
            // Fetch user profile data
            const profileData = await fetchUserProfile(authUser.id);
            setUser(profileData);
          }
        } else {
          setSupabaseUser(null);
          setSession(null);
          setUser(null);
        }
        
        setLoading(false);
        
        // Force a router refresh to update server components
        if (event === 'SIGNED_IN' || event === 'SIGNED_OUT') {
          router.refresh();
        }
      }
    );

    return () => {
      subscription.unsubscribe();
    };
  }, [supabase, router]);

  const signOut = async () => {
    await supabase.auth.signOut();
    router.push('/');
  };

  return (
    <AuthContext.Provider value={{ 
      session, 
      supabaseUser, 
      user, 
      loading, 
      signOut 
    }}>
      {children}
    </AuthContext.Provider>
  );
};

export const useAuth = () => {
  const context = useContext(AuthContext);
  if (context === undefined) {
    throw new Error('useAuth must be used within an AuthProvider');
  }
  return context;
};
